"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseCliArgs = parseCliArgs;
const error_1 = require("@pnpm/error");
const find_workspace_dir_1 = require("@pnpm/find-workspace-dir");
const nopt_1 = __importDefault(require("@pnpm/nopt"));
const didyoumean2_1 = __importStar(require("didyoumean2"));
const RECURSIVE_CMDS = new Set(['recursive', 'multi', 'm']);
async function parseCliArgs(opts, inputArgv) {
    const noptExploratoryResults = (0, nopt_1.default)({
        filter: [String],
        help: Boolean,
        recursive: Boolean,
        ...opts.universalOptionsTypes,
        ...opts.getTypesByCommandName('add'),
        ...opts.getTypesByCommandName('install'),
    }, {
        r: '--recursive',
        ...opts.universalShorthands,
    }, inputArgv, 0, { escapeArgs: opts.escapeArgs });
    const recursiveCommandUsed = RECURSIVE_CMDS.has(noptExploratoryResults.argv.remain[0]);
    let commandName = getCommandName(noptExploratoryResults.argv.remain);
    let cmd = commandName ? opts.getCommandLongName(commandName) : null;
    const fallbackCommandUsed = Boolean(commandName && !cmd && opts.fallbackCommand);
    if (fallbackCommandUsed) {
        cmd = opts.fallbackCommand;
        commandName = opts.fallbackCommand;
        inputArgv.unshift(opts.fallbackCommand);
        // The run command has special casing for --help and is handled further below.
    }
    else if (cmd !== 'run') {
        if (noptExploratoryResults['help']) {
            return {
                ...getParsedArgsForHelp(),
                workspaceDir: await getWorkspaceDir(noptExploratoryResults),
            };
        }
        if (noptExploratoryResults['version'] || noptExploratoryResults['v']) {
            return {
                argv: noptExploratoryResults.argv,
                cmd: null,
                options: {
                    version: true,
                },
                params: noptExploratoryResults.argv.remain,
                unknownOptions: new Map(),
                fallbackCommandUsed: false,
                workspaceDir: await getWorkspaceDir(noptExploratoryResults),
            };
        }
    }
    function getParsedArgsForHelp() {
        return {
            argv: noptExploratoryResults.argv,
            cmd: 'help',
            options: {},
            params: noptExploratoryResults.argv.remain,
            unknownOptions: new Map(),
            fallbackCommandUsed: false,
        };
    }
    const types = {
        ...opts.universalOptionsTypes,
        ...opts.getTypesByCommandName(commandName),
    }; // eslint-disable-line @typescript-eslint/no-explicit-any
    function getCommandName(args) {
        if (recursiveCommandUsed) {
            args = args.slice(1);
        }
        if (opts.getCommandLongName(args[0]) !== 'install' || args.length === 1) {
            return args[0];
        }
        return 'add';
    }
    function getEscapeArgsWithSpecialCaseForRun() {
        if (cmd !== 'run') {
            return opts.escapeArgs;
        }
        // We'd like everything after the run script's name to be passed to the
        // script's argv itself. For example, "pnpm run echo --test" should pass
        // "--test" to the "echo" script. This requires determining the script's
        // name and declaring it as the "escape arg".
        //
        // The name of the run script is normally the second argument (ex: pnpm
        // run foo), but can be pushed back by recursive commands (ex: pnpm
        // recursive run foo) or becomes the first argument when the fallback
        // command (ex: pnpm foo) is set to 'run'.
        const indexOfRunScriptName = 1 +
            (recursiveCommandUsed ? 1 : 0) +
            (fallbackCommandUsed && opts.fallbackCommand === 'run' ? -1 : 0);
        return [noptExploratoryResults.argv.remain[indexOfRunScriptName]];
    }
    const { argv, ...options } = (0, nopt_1.default)({
        recursive: Boolean,
        ...types,
    }, {
        ...opts.universalShorthands,
        ...opts.shorthandsByCommandName[commandName],
    }, inputArgv, 0, { escapeArgs: getEscapeArgsWithSpecialCaseForRun() });
    const workspaceDir = await getWorkspaceDir(options);
    // For the run command, it's not clear whether --help should be passed to the
    // underlying script or invoke pnpm's help text until an additional nopt call.
    if (cmd === 'run' && options['help']) {
        return {
            ...getParsedArgsForHelp(),
            workspaceDir,
        };
    }
    if (opts.renamedOptions != null) {
        for (const [cliOption, optionValue] of Object.entries(options)) {
            if (opts.renamedOptions[cliOption]) {
                options[opts.renamedOptions[cliOption]] = optionValue;
                delete options[cliOption];
            }
        }
    }
    const params = argv.remain.slice(1);
    if (options['recursive'] !== true && (options['filter'] || options['filter-prod'] || recursiveCommandUsed)) {
        options['recursive'] = true;
        const subCmd = argv.remain[1] && opts.getCommandLongName(argv.remain[1]);
        if (subCmd && recursiveCommandUsed) {
            params.shift();
            argv.remain.shift();
            cmd = subCmd;
        }
    }
    if (options['workspace-root']) {
        if (options['global']) {
            throw new error_1.PnpmError('OPTIONS_CONFLICT', '--workspace-root may not be used with --global');
        }
        if (!workspaceDir) {
            throw new error_1.PnpmError('NOT_IN_WORKSPACE', '--workspace-root may only be used inside a workspace');
        }
        options['dir'] = workspaceDir;
    }
    if (cmd === 'install' && params.length > 0) {
        cmd = 'add';
    }
    else if (!cmd && options['recursive']) {
        cmd = 'recursive';
    }
    const knownOptions = new Set(Object.keys(types));
    return {
        argv,
        cmd,
        params,
        workspaceDir,
        fallbackCommandUsed,
        ...normalizeOptions(options, knownOptions),
    };
}
const CUSTOM_OPTION_PREFIX = 'config.';
function normalizeOptions(options, knownOptions) {
    const standardOptionNames = [];
    const normalizedOptions = {};
    for (const [optionName, optionValue] of Object.entries(options)) {
        if (optionName.startsWith(CUSTOM_OPTION_PREFIX)) {
            normalizedOptions[optionName.substring(CUSTOM_OPTION_PREFIX.length)] = optionValue;
            continue;
        }
        normalizedOptions[optionName] = optionValue;
        standardOptionNames.push(optionName);
    }
    const unknownOptions = getUnknownOptions(standardOptionNames, knownOptions);
    return { options: normalizedOptions, unknownOptions };
}
function getUnknownOptions(usedOptions, knownOptions) {
    const unknownOptions = new Map();
    const closestMatches = getClosestOptionMatches.bind(null, Array.from(knownOptions));
    for (const usedOption of usedOptions) {
        if (knownOptions.has(usedOption) || usedOption.startsWith('//'))
            continue;
        unknownOptions.set(usedOption, closestMatches(usedOption));
    }
    return unknownOptions;
}
function getClosestOptionMatches(knownOptions, option) {
    return (0, didyoumean2_1.default)(option, knownOptions, {
        returnType: didyoumean2_1.ReturnTypeEnums.ALL_CLOSEST_MATCHES,
    });
}
async function getWorkspaceDir(parsedOpts) {
    if (parsedOpts['global'] || parsedOpts['ignore-workspace'])
        return undefined;
    const dir = parsedOpts['dir'] ?? process.cwd();
    return (0, find_workspace_dir_1.findWorkspaceDir)(dir);
}
//# sourceMappingURL=index.js.map